import 'dart:async';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/access_management/booking_system/domain/models/bookable_room.dart';
import 'package:syncrow_web/pages/access_management/booking_system/domain/services/booking_system_service.dart';
import 'package:syncrow_web/pages/access_management/booking_system/presentation/bloc/sidebar/sidebar_event.dart';
import 'package:syncrow_web/pages/access_management/booking_system/presentation/bloc/sidebar/sidebar_state.dart';

class SidebarBloc extends Bloc<SidebarEvent, SidebarState> {
  final BookingSystemService _bookingService;
  Timer? _searchDebounce;
  int _currentPage = 1;
  final int _pageSize = 20;
  String _currentSearch = '';

  SidebarBloc(this._bookingService)
      : super(SidebarState(
          allRooms: [],
          displayedRooms: [],
          isLoading: true,
          hasMore: true,
        )) {
    on<LoadBookableSpaces>(_onLoadBookableSpaces);
    on<LoadMoreSpaces>(_onLoadMoreSpaces);
    on<SelectRoomEvent>(_onSelectRoom);
    on<SearchRoomsEvent>(_onSearchRooms);
    on<ResetSearch>(_onResetSearch);
  }

  Future<void> _onLoadBookableSpaces(
    LoadBookableSpaces event,
    Emitter<SidebarState> emit,
  ) async {
    try {
      emit(state.copyWith(isLoading: true, errorMessage: null));
      _currentPage = 1;
      _currentSearch = '';

      final paginatedSpaces = await _bookingService.getBookableSpaces(
        page: _currentPage,
        size: _pageSize,
        search: _currentSearch,
      );

      final rooms = paginatedSpaces.data.map((space) {
        return BookableSpaceModel(
          uuid: space.uuid,
          spaceName: space.spaceName,
          virtualLocation: space.virtualLocation,
          bookableConfig: BookableConfig(
            uuid: space.bookableConfig.uuid,
            daysAvailable: space.bookableConfig.daysAvailable,
            startTime: space.bookableConfig.startTime,
            endTime: space.bookableConfig.endTime,
            active: space.bookableConfig.active,
            points: space.bookableConfig.points,
          ),
        );
      }).toList();

      emit(state.copyWith(
        allRooms: rooms,
        displayedRooms: rooms,
        isLoading: false,
        hasMore: paginatedSpaces.hasNext,
        totalPages: paginatedSpaces.totalPage,
        currentPage: _currentPage,
      ));
    } catch (e) {
      emit(state.copyWith(
        isLoading: false,
        errorMessage: 'Failed to load rooms: ${e.toString()}',
      ));
    }
  }

  Future<void> _onLoadMoreSpaces(
    LoadMoreSpaces event,
    Emitter<SidebarState> emit,
  ) async {
    if (!state.hasMore || state.isLoadingMore) return;

    try {
      emit(state.copyWith(isLoadingMore: true));
      _currentPage++;

      final paginatedSpaces = await _bookingService.getBookableSpaces(
        page: _currentPage,
        size: _pageSize,
        search: _currentSearch,
      );

      final newRooms = paginatedSpaces.data.map((space) {
        return BookableSpaceModel(
          uuid: space.uuid,
          spaceName: space.spaceName,
          virtualLocation: space.virtualLocation,
          bookableConfig: BookableConfig(
            uuid: space.bookableConfig.uuid,
            daysAvailable: space.bookableConfig.daysAvailable,
            startTime: space.bookableConfig.startTime,
            endTime: space.bookableConfig.endTime,
            active: space.bookableConfig.active,
            points: space.bookableConfig.points,
          ),
        );
      }).toList();

      final updatedRooms = [...state.allRooms, ...newRooms];

      emit(state.copyWith(
        allRooms: updatedRooms,
        displayedRooms: updatedRooms,
        isLoadingMore: false,
        hasMore: paginatedSpaces.hasNext,
        currentPage: _currentPage,
      ));
    } catch (e) {
      _currentPage--;
      emit(state.copyWith(
        isLoadingMore: false,
        errorMessage: 'Failed to load more rooms: ${e.toString()}',
      ));
    }
  }

  Future<void> _onSearchRooms(
    SearchRoomsEvent event,
    Emitter<SidebarState> emit,
  ) async {
    try {
      _currentSearch = event.query;
      _currentPage = 1;
      emit(state.copyWith(isLoading: true, errorMessage: null));
      final paginatedSpaces = await _bookingService.getBookableSpaces(
        page: _currentPage,
        size: _pageSize,
        search: _currentSearch,
      );
      final rooms = paginatedSpaces.data.map((space) {
        return BookableSpaceModel(
          uuid: space.uuid,
          spaceName: space.spaceName,
          virtualLocation: space.virtualLocation,
          bookableConfig: BookableConfig(
            uuid: space.bookableConfig.uuid,
            daysAvailable: space.bookableConfig.daysAvailable,
            startTime: space.bookableConfig.startTime,
            endTime: space.bookableConfig.endTime,
            active: space.bookableConfig.active,
            points: space.bookableConfig.points,
          ),
        );
      }).toList();
      emit(state.copyWith(
        allRooms: rooms,
        displayedRooms: rooms,
        isLoading: false,
        hasMore: paginatedSpaces.hasNext,
        totalPages: paginatedSpaces.totalPage,
        currentPage: _currentPage,
      ));
    } catch (e) {
      emit(state.copyWith(
        isLoading: false,
        errorMessage: 'Search failed: ${e.toString()}',
      ));
    }
  }

  void _onResetSearch(
    ResetSearch event,
    Emitter<SidebarState> emit,
  ) {
    _currentSearch = '';
    add(LoadBookableSpaces());
  }

  void _onSelectRoom(
    SelectRoomEvent event,
    Emitter<SidebarState> emit,
  ) {
    emit(state.copyWith(selectedRoomId: event.roomId));
  }

  @override
  Future<void> close() {
    _searchDebounce?.cancel();
    return super.close();
  }
}
