import 'dart:async';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:syncrow_web/pages/access_management/booking_system/data/services/bookable_spaces_service.dart';
import 'package:syncrow_web/pages/access_management/booking_system/domain/models/bookable_room.dart';
import 'package:syncrow_web/pages/access_management/booking_system/presentation/bloc/sidebar/sidebar_bloc.dart';
import 'package:syncrow_web/pages/access_management/booking_system/presentation/bloc/sidebar/sidebar_event.dart';
import 'package:syncrow_web/pages/access_management/booking_system/presentation/bloc/sidebar/sidebar_state.dart';
import 'package:syncrow_web/pages/access_management/booking_system/presentation/view/widgets/room_list_item.dart';
import 'package:syncrow_web/services/api/http_service.dart';
import 'package:syncrow_web/utils/color_manager.dart';
import 'package:syncrow_web/utils/constants/assets.dart';

class BookingSidebar extends StatelessWidget {
  final void Function(BookableSpaceModel) onRoomSelected;

  const BookingSidebar({
    super.key,
    required this.onRoomSelected,
  });

  @override
  Widget build(BuildContext context) {
    return BlocProvider(
      create: (context) => SidebarBloc(BookableSpacesService(
        HTTPService(),
      ))
        ..add(LoadBookableSpaces()),
      child: _SidebarContent(onRoomSelected: onRoomSelected),
    );
  }
}

class _SidebarContent extends StatefulWidget {
  final void Function(BookableSpaceModel) onRoomSelected;

  const _SidebarContent({
    required this.onRoomSelected,
  });

  @override
  State<_SidebarContent> createState() => __SidebarContentState();
}

class __SidebarContentState extends State<_SidebarContent> {
  final TextEditingController searchController = TextEditingController();
  final ScrollController _scrollController = ScrollController();
  Timer? _searchDebounce;

  @override
  void initState() {
    super.initState();
    _scrollController.addListener(_scrollListener);
  }

  @override
  void dispose() {
    _scrollController.dispose();
    _searchDebounce?.cancel();
    super.dispose();
  }

  void _scrollListener() {
    if (_scrollController.position.pixels ==
        _scrollController.position.maxScrollExtent) {
      context.read<SidebarBloc>().add(LoadMoreSpaces());
    }
  }

  void _handleSearch(String value) {
    _searchDebounce?.cancel();
    _searchDebounce = Timer(const Duration(milliseconds: 300), () {
      context.read<SidebarBloc>().add(SearchRoomsEvent(value));
    });
  }

  @override
  Widget build(BuildContext context) {
    return BlocConsumer<SidebarBloc, SidebarState>(
      listener: (context, state) {
        if (state.currentPage == 1 && searchController.text.isNotEmpty) {
          searchController.clear();
        }
      },
      builder: (context, state) {
        return Column(
          children: [
            const _SidebarHeader(title: 'Spaces'),
            Container(
              decoration: BoxDecoration(
                color: ColorsManager.whiteColors,
                borderRadius: BorderRadius.circular(8.0),
                boxShadow: [
                  BoxShadow(
                    color: ColorsManager.blackColor.withOpacity(0.1),
                    offset: const Offset(0, -2),
                    blurRadius: 4,
                    spreadRadius: 0,
                  ),
                  BoxShadow(
                    color: ColorsManager.blackColor.withOpacity(0.1),
                    offset: const Offset(0, 2),
                    blurRadius: 4,
                    spreadRadius: 0,
                  ),
                ],
              ),
              child: Padding(
                padding: const EdgeInsets.all(8.0),
                child: Container(
                  child: Padding(
                    padding: const EdgeInsets.symmetric(
                        horizontal: 16.0, vertical: 8.0),
                    child: Container(
                      decoration: BoxDecoration(
                        color: ColorsManager.counterBackgroundColor,
                        borderRadius: BorderRadius.circular(8.0),
                      ),
                      child: Row(
                        children: [
                          Expanded(
                            child: TextField(
                              style: Theme.of(context)
                                  .textTheme
                                  .bodyMedium
                                  ?.copyWith(
                                    color: ColorsManager.blackColor,
                                  ),
                              controller: searchController,
                              onChanged: _handleSearch,
                              decoration: InputDecoration(
                                hintText: 'Search',
                                suffixIcon: Padding(
                                  padding: const EdgeInsets.all(8.0),
                                  child: SizedBox(
                                    width: 20,
                                    height: 20,
                                    child: SvgPicture.asset(
                                      Assets.searchIconUser,
                                      color: ColorsManager.primaryTextColor,
                                    ),
                                  ),
                                ),
                                contentPadding: const EdgeInsets.symmetric(
                                    vertical: 8, horizontal: 12),
                                border: const OutlineInputBorder(
                                    borderSide: BorderSide.none),
                              ),
                            ),
                          ),
                          if (searchController.text.isNotEmpty)
                            IconButton(
                              icon: const Icon(Icons.close),
                              onPressed: () {
                                context.read<SidebarBloc>().add(ResetSearch());
                              },
                            ),
                        ],
                      ),
                    ),
                  ),
                ),
              ),
            ),
            if (state.isLoading)
              const Expanded(
                child: Center(child: CircularProgressIndicator()),
              )
            else if (state.errorMessage != null)
              Expanded(
                child: Center(child: Text(state.errorMessage!)),
              )
            else
              Expanded(
                child: ListView.builder(
                  controller: _scrollController,
                  itemCount:
                      state.displayedRooms.length + (state.hasMore ? 1 : 0),
                  itemBuilder: (context, index) {
                    if (index == state.displayedRooms.length) {
                      return _buildLoadMoreIndicator(state);
                    }

                    final room = state.displayedRooms[index];
                    return RoomListItem(
                      room: room,
                      isSelected: state.selectedRoomId == room.uuid,
                      onTap: () {
                        context
                            .read<SidebarBloc>()
                            .add(SelectRoomEvent(room.uuid));
                        widget.onRoomSelected(room);
                      },
                    );
                  },
                ),
              ),
          ],
        );
      },
    );
  }

  Widget _buildLoadMoreIndicator(SidebarState state) {
    if (state.isLoadingMore) {
      return const Padding(
        padding: EdgeInsets.symmetric(vertical: 16.0),
        child: Center(child: CircularProgressIndicator()),
      );
    } else if (state.hasMore) {
      return const Padding(
        padding: EdgeInsets.symmetric(vertical: 16.0),
        child: Center(child: Text('Scroll to load more')),
      );
    } else {
      return const SizedBox.shrink();
    }
  }
}

class _SidebarHeader extends StatelessWidget {
  final String title;

  const _SidebarHeader({
    required this.title,
  });

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.all(16.0),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          Text(
            title,
            style: Theme.of(context).textTheme.titleLarge?.copyWith(
                  fontWeight: FontWeight.w400,
                  color: ColorsManager.primaryTextColor,
                  fontSize: 20,
                ),
          ),
        ],
      ),
    );
  }
}
