import 'package:flutter/material.dart';
import 'package:calendar_view/calendar_view.dart';
import 'package:intl/intl.dart';
import 'package:syncrow_web/utils/color_manager.dart';

class WeeklyCalendarPage extends StatelessWidget {
  final DateTime weekStart;
  final DateTime selectedDate;
  final EventController eventController;
  final String? startTime;
  final String? endTime;

  const WeeklyCalendarPage({
    super.key,
    required this.weekStart,
    required this.selectedDate,
    required this.eventController,
    this.startTime,
    this.endTime,
  });

  @override
  Widget build(BuildContext context) {
    final startHour = _parseHour(startTime, defaultValue: 0);
    final endHour = _parseHour(endTime, defaultValue: 24);
    if (endTime == null || endTime!.isEmpty) {
      return const Center(
        child: Text(
          'Please select a bookable space to view the calendar.',
          style: TextStyle(fontSize: 16, fontWeight: FontWeight.w500),
        ),
      );
    }

    final weekDays = _getWeekDays(weekStart);

    return LayoutBuilder(
      builder: (context, constraints) {
        final double calendarWidth = constraints.maxWidth;
        const double timeLineWidth = 80;
        const int totalDays = 7;
        final double dayColumnWidth =
            (calendarWidth - timeLineWidth) / totalDays;
        final selectedDayIndex =
            weekDays.indexWhere((d) => isSameDay(d, selectedDate));
        return Padding(
          padding: const EdgeInsets.only(left: 25.0, right: 25.0, top: 25),
          child: Stack(
            children: [
              WeekView(
                key: ValueKey(weekStart),
                controller: eventController,
                initialDay: weekStart,
                startHour: startHour - 1,
                endHour: endHour,
                heightPerMinute: 1.1,
                showLiveTimeLineInAllDays: false,
                showVerticalLines: true,
                emulateVerticalOffsetBy: -80,
                startDay: WeekDays.monday,
                liveTimeIndicatorSettings: const LiveTimeIndicatorSettings(
                  showBullet: false,
                  height: 0,
                ),
                weekDayBuilder: (date) {
                  final weekDays = _getWeekDays(weekStart);
                  final selectedDayIndex =
                      weekDays.indexWhere((d) => isSameDay(d, selectedDate));
                  final index = weekDays.indexWhere((d) => isSameDay(d, date));
                  final isSelectedDay = index == selectedDayIndex;
                  final isToday = isSameDay(date, DateTime.now());

                  return Container(
                    decoration: isSelectedDay
                        ? BoxDecoration(
                            color: ColorsManager.blue1.withOpacity(0.2),
                            borderRadius: BorderRadius.circular(6),
                          )
                        : isToday
                            ? BoxDecoration(
                                color: ColorsManager.blue1.withOpacity(0.08),
                                borderRadius: BorderRadius.circular(6),
                              )
                            : null,
                    child: Column(
                      children: [
                        Text(
                          DateFormat('EEE').format(date).toUpperCase(),
                          style: TextStyle(
                            fontWeight: FontWeight.w400,
                            fontSize: 14,
                            color: isSelectedDay ? Colors.blue : Colors.black,
                          ),
                        ),
                        Text(
                          DateFormat('d').format(date),
                          style: TextStyle(
                            fontWeight: FontWeight.w700,
                            fontSize: 20,
                            color: isSelectedDay
                                ? ColorsManager.blue1
                                : ColorsManager.blackColor,
                          ),
                        ),
                      ],
                    ),
                  );
                },
                timeLineBuilder: (date) {
                  int hour = date.hour == 0
                      ? 12
                      : (date.hour > 12 ? date.hour - 12 : date.hour);
                  String period = date.hour >= 12 ? 'PM' : 'AM';
                  return Container(
                    height: 60,
                    alignment: Alignment.center,
                    child: RichText(
                      text: TextSpan(
                        children: [
                          TextSpan(
                            text: '$hour',
                            style: const TextStyle(
                              fontWeight: FontWeight.w700,
                              fontSize: 24,
                              color: ColorsManager.blackColor,
                            ),
                          ),
                          WidgetSpan(
                            child: Padding(
                              padding: const EdgeInsets.only(left: 2, top: 6),
                              child: Text(
                                period,
                                style: const TextStyle(
                                  fontWeight: FontWeight.w400,
                                  fontSize: 12,
                                  color: ColorsManager.blackColor,
                                  letterSpacing: 1,
                                ),
                              ),
                            ),
                            alignment: PlaceholderAlignment.baseline,
                            baseline: TextBaseline.alphabetic,
                          ),
                        ],
                      ),
                    ),
                  );
                },
                timeLineWidth: timeLineWidth,
                weekPageHeaderBuilder: (start, end) => Container(),
                weekTitleHeight: 60,
                weekNumberBuilder: (firstDayOfWeek) => Text(
                  firstDayOfWeek.timeZoneName,
                  style: Theme.of(context).textTheme.bodyMedium?.copyWith(
                        color: Colors.black,
                        fontWeight: FontWeight.bold,
                      ),
                ),
                eventTileBuilder: (date, events, boundary, start, end) {
                  return Container(
                    margin:
                        const EdgeInsets.symmetric(vertical: 2, horizontal: 2),
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: events.map((event) {
                        final bool isEventEnded = event.endTime != null &&
                            event.endTime!.isBefore(DateTime.now());
                        return Expanded(
                          child: Container(
                            width: double.infinity,
                            padding: const EdgeInsets.all(6),
                            decoration: BoxDecoration(
                              color: isEventEnded
                                  ? ColorsManager.grayColor
                                  : ColorsManager.lightGrayColor
                                      .withOpacity(0.25),
                              borderRadius: BorderRadius.circular(6),
                            ),
                            child: Column(
                              crossAxisAlignment: CrossAxisAlignment.start,
                              children: [
                                Text(
                                  DateFormat('h:mm a').format(event.startTime!),
                                  style: const TextStyle(
                                    fontWeight: FontWeight.bold,
                                    fontSize: 12,
                                    color: Colors.black87,
                                  ),
                                ),
                                const SizedBox(height: 2),
                                Text(
                                  event.title,
                                  style: const TextStyle(
                                    fontSize: 12,
                                    color: ColorsManager.blackColor,
                                  ),
                                ),
                              ],
                            ),
                          ),
                        );
                      }).toList(),
                    ),
                  );
                },
              ),
              if (selectedDayIndex >= 0)
                Positioned(
                  left: timeLineWidth + dayColumnWidth * selectedDayIndex,
                  top: 0,
                  bottom: 0,
                  width: dayColumnWidth,
                  child: IgnorePointer(
                    child: Container(
                      margin: const EdgeInsets.symmetric(
                          vertical: 0, horizontal: 2),
                      color: ColorsManager.blue1.withOpacity(0.1),
                    ),
                  ),
                ),
              Positioned(
                right: 0,
                top: 50,
                bottom: 0,
                child: IgnorePointer(
                  child: Container(
                    width: 1,
                    color: Theme.of(context).scaffoldBackgroundColor,
                  ),
                ),
              ),
            ],
          ),
        );
      },
    );
  }

  List<DateTime> _getWeekDays(DateTime date) {
    final int weekday = date.weekday;
    final DateTime monday = date.subtract(Duration(days: weekday - 1));
    return List.generate(7, (i) => monday.add(Duration(days: i)));
  }
}

bool isSameDay(DateTime d1, DateTime d2) {
  return d1.year == d2.year && d1.month == d2.month && d1.day == d2.day;
}

int _parseHour(String? time, {required int defaultValue}) {
  if (time == null || time.isEmpty || !time.contains(':')) {
    return defaultValue;
  }
  try {
    return int.parse(time.split(':')[0]);

  } catch (e) {
    return defaultValue;
  }
}
