import 'package:flutter/material.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/tags/domain/models/tag.dart';
import 'package:syncrow_web/utils/color_manager.dart';
import 'package:syncrow_web/utils/extension/build_context_x.dart';

class ProductTagField extends StatefulWidget {
  final List<Tag> items;
  final ValueChanged<Tag> onSelected;
  final Tag? initialValue;
  final String productName;

  const ProductTagField({
    super.key,
    required this.items,
    required this.onSelected,
    this.initialValue,
    required this.productName,
  });

  @override
  State<ProductTagField> createState() => _ProductTagFieldState();
}

class _ProductTagFieldState extends State<ProductTagField> {
  bool _isOpen = false;
  OverlayEntry? _overlayEntry;
  final TextEditingController _controller = TextEditingController();
  final FocusNode _focusNode = FocusNode();

  @override
  void initState() {
    super.initState();
    _controller.text = widget.initialValue?.name ?? '';
    _focusNode.addListener(_handleFocusChange);
  }

  @override
  void dispose() {
    _focusNode.removeListener(_handleFocusChange);
    _controller.dispose();
    _focusNode.dispose();
    _overlayEntry?.remove();
    _overlayEntry = null;
    super.dispose();
  }

  void _handleFocusChange() {
    if (!_focusNode.hasFocus) {
      _submit(_controller.text);
    }
  }

  void _submit(String value) {
    final lowerCaseValue = value.toLowerCase();
    final selectedTag = widget.items.firstWhere(
      (tag) => tag.name.toLowerCase() == lowerCaseValue,
      orElse: () => Tag(
        name: value,
        uuid: '',
        createdAt: '',
        updatedAt: '',
      ),
    );
    widget.onSelected(selectedTag);
    _closeDropdown();
  }

  void _toggleDropdown() {
    if (_isOpen) {
      _closeDropdown();
    } else {
      _openDropdown();
    }
  }

  void _openDropdown() {
    _overlayEntry = _createOverlayEntry();
    Overlay.of(context).insert(_overlayEntry!);
    setState(() => _isOpen = true);
  }

  void _closeDropdown() {
    if (_isOpen) {
      _overlayEntry?.remove();
      _overlayEntry = null;
      setState(() => _isOpen = false);
    }
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () => FocusScope.of(context).unfocus(),
      behavior: HitTestBehavior.opaque,
      child: Container(
        padding: const EdgeInsets.symmetric(horizontal: 12.0, vertical: 8.0),
        decoration: BoxDecoration(
          border: Border.all(color: ColorsManager.transparentColor),
          borderRadius: BorderRadius.circular(8.0),
        ),
        child: Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: [
            Expanded(
              child: TextFormField(
                controller: _controller,
                focusNode: _focusNode,
                onFieldSubmitted: _submit,
                style: context.textTheme.bodyMedium,
                decoration: const InputDecoration(
                  hintText: 'Enter or Select a tag',
                  border: InputBorder.none,
                ),
              ),
            ),
            GestureDetector(
              onTap: _toggleDropdown,
              child: const Icon(Icons.arrow_drop_down),
            ),
          ],
        ),
      ),
    );
  }

  OverlayEntry _createOverlayEntry() {
    final renderBox = context.findRenderObject()! as RenderBox;
    final size = renderBox.size;
    final offset = renderBox.localToGlobal(Offset.zero);

    return OverlayEntry(
      builder: (context) {
        return GestureDetector(
          onTap: _closeDropdown,
          behavior: HitTestBehavior.translucent,
          child: Stack(
            children: [
              Positioned(
                left: offset.dx,
                top: offset.dy + size.height,
                width: size.width,
                child: Material(
                  elevation: 4.0,
                  child: Container(
                    color: ColorsManager.whiteColors,
                    constraints: const BoxConstraints(maxHeight: 200.0),
                    child: ListView.builder(
                      shrinkWrap: true,
                      itemCount: widget.items.length,
                      itemBuilder: (context, index) {
                        final tag = widget.items[index];
                        return Container(
                          decoration: const BoxDecoration(
                            border: Border(
                              bottom: BorderSide(
                                color: ColorsManager.lightGrayBorderColor,
                                width: 1.0,
                              ),
                            ),
                          ),
                          child: ListTile(
                            title: Text(
                              tag.name,
                              style: context.textTheme.bodyMedium?.copyWith(
                                color: ColorsManager.textPrimaryColor,
                              ),
                            ),
                            onTap: () {
                              _controller.text = tag.name;
                              _submit(tag.name);
                              _closeDropdown();
                            },
                          ),
                        );
                      },
                    ),
                  ),
                ),
              ),
            ],
          ),
        );
      },
    );
  }
}
